#ifndef _SPLINE_
#define _SPLINE_	

#include <vector>	
#include "VVector.h"
#include "TreeEntity.h"



/**
* class representing an entity for a family tree
*/
class Spline
{

public:

	/**
	* standard constructor
	*/
	Spline() :
	  m_fAlpha( 0.0f ),
	  m_fBeta( 0.0f ),
	  m_fEmphasize( 0.0f ),
	  m_bStatus( true ),
	  m_vColorStart( NULL), m_vColorEnd( NULL ),
	  m_LongestSpline( 2 )
	{
	}

	/**
	* constructor
	*/
	Spline( std::vector< TreeEntity* > pts, VVector *cs, VVector *ce, float alpha, float beta ) :
		points( pts ),
		m_fAlpha( alpha ),
		m_fBeta( beta ),
		m_bStatus( true ),
		m_vColorStart( cs ),
		m_vColorEnd( ce ),
		m_LongestSpline( 2 ),
		m_fEmphasize( 0.5f )
		
	{
		
	}

	/**
	* standard destructor
	*/
	~Spline() 
	{
		points.clear();
	}

	/**
	* sets the alpha transparency parameter
	* @param alpha is the alpha parameter
	*/
	void setAlpha ( float alpha )
	{
		m_fAlpha = alpha;
	}

	/**
	* sets longest spline
	* @param num is the maximal number of points that occur
	*/
	void setLongest ( int num )
	{
		m_LongestSpline = num;
	}

	/**
	* sets the beta shift parameter
	* @param beta is the beta shift parameter
	*/
	void setBeta ( float beta )
	{
		m_fBeta = beta;
	}

	/**
	* sets the emphasize parameter
	* @param emph is the emphasize parameter
	*/
	void setEmphasize ( float emph )
	{
		m_fEmphasize = emph;
	}
	
	/**
	* sets the status on/off
	* @param status is the status parameter
	*/
	void setStatus ( bool status )
	{
		m_bStatus = status;
	}

	/**
	* sets the color of the starting of the spline
	* @param cs is the starting color
	*/
	void setColorStart ( VVector *cs )
	{
		m_vColorStart = cs;
	}

	/**
	* sets the color of the ending of the spline
	* @param ce is the ending color
	*/
	void setColorEnd ( VVector *ce )
	{
		m_vColorEnd = ce;
	}

	/**
	* returns true if this spline has an intersection with a segment (abscisse) p1 - p2
	* @param p1 is the starting point of the segment
	* @param p2 is the ending point of the segment
	* @return true if this spline has an intersection with a segment (abscisse) p1 - p2
	*/
	bool hasIntersection( float *line );

	/**
	* returns true if this spline has an intersection with a segment (abscisse) p1 - p2 for inverse rooted trees (extra)
	* @param p1 is the starting point of the segment
	* @param p2 is the ending point of the segment
	* @return true if this spline has an intersection with a segment (abscisse) p1 - p2
	*/
	bool hasIntersectionSpecial( float *line );

	/**
	* draws this spine
	*/
	void draw(  );

	void drawInvRooted( );


private:

	/**
	* returns a basis of a b-spline b_i_n(t)
	* @param i is int i in b_i_n(t)
	* @param n is int n in b_i_n(t)
	* @param t is float t in b_i_n(t)
	* @param vknots is the knot vector
	* @return float basis b_i_n(t)
	*/
	float getBasis( int i, int n, float t, std::vector< float > *vknots );

	//spline points
	std::vector< TreeEntity* > points;	/**< vector data structure containing the 2d control points. */
	std::vector< VVector > rasterized;	/**< vector data structure containing the splines raster. */

	//spline properties
	float m_fBeta;				/**< beta - shift parameter, ranges from 0 to 1. */
	float m_fAlpha;				/**< alpha - transparency parameter, ranges from 0 to 1. */
	float m_fEmphasize;			/**< emphasize - transparency parameter, ranges from 0 to 1. */
	bool m_bStatus;				/**< true iff the spline is visible. */
	VVector *m_vColorStart;		/**< RGB color of the starting point of the spline. */
	VVector *m_vColorEnd;		/**< RGB color of the ending point of the spline. */

	int m_LongestSpline; 		/**< Number of control point of the longest spline - for alpha mapping. */

};

#endif