#ifndef QVOLRENDCANVAS_H_
#define QVOLRENDCANVAS_H_

#include <Qt/qobject.h>

#include <QtGui/QWidget>
#include <QtOpenGL/QGLWidget>
#include <QtGui/QKeyEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QTextEdit>
#include "QFotoCanvas.h"

#include "VEvent.h"
#include "TreeRep.h"


/**
 * enum for handling mousestates
 */
enum MouseState
{
	NO_DRAGGING = 0,		/**< Mouse is up */
	ZOOM,					/**< Right Mouse is down */
	PAN,					/**< Left Mouse is down */
};

/**
 * enum for handling tree layout changes
 */
enum TreeLayout
{
	LAY_RADIAL = 0,
	LAY_INVRADIAL	=  1,
	LAY_ROOTED	=  2,
	LAY_INVROOTED =  3,
	LAY_BALLOON = 4
};

/**
 * class representing the Main render Canvas
 */
class QVolRendCanvas : public QObject, public QGLWidget
{
	Q_OBJECT

public:

	/**
	 * constructor with given parent Widget
	 * @param parent Widget
	 */
	QVolRendCanvas(QWidget* parent = 0);
	
	/**
	 * standard destructor
	 */
	virtual ~QVolRendCanvas();

	/**
	 * sets the name of the current Object
	 * @param name QString containing the widgets name
	 */
	void setObjectName(const QString &name);

	/**
	 * method called when resizing
	 * @param w width of window
	 * @param h height of window
	 */
    virtual void resizeGL( int w, int h );

	/**
	 * method handling the paintEvent
	 * @param e Pointer to QPaintEvent
	 */
	virtual void paintEvent( QPaintEvent *e );

	/**
	 * method handling the mousePressEvent
	 * @param e Pointer to QMouseEvent
	 */
	virtual void mousePressEvent ( QMouseEvent * e );

	/**
	* method handling the mouseReleaseEvent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseReleaseEvent ( QMouseEvent * e );

	/**
	* method handling the mouseMoveEvent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseMoveEvent ( QMouseEvent * e );


	/**
	* method handling the mouseDoubleClickEvent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseDoubleClickEvent ( QMouseEvent * e );


	/**
	* method handling the keyPressEvent
	* @param e Pointer to QKeyEvent
	*/
	virtual void keyPressEvent ( QKeyEvent * e );


	/**
	* method handling the keyReleaseEvent
	* @param e Pointer to QKeyEvent
	*/
	virtual void keyReleaseEvent ( QKeyEvent * e );

	/**
	 * method that converts a QMouseEvent into a VMouseEvent
	 * @param e Ptr to QMouseEvent
	 * @return VMouseEvent
	 */
	const VMouseEvent getMouseEvent (QMouseEvent *e);

	/**
	 * method that converts a QKeyEvent into a VKeybardEvent
	 * @param e Ptr to QMouseEvent
	 * @return VKeyboardEvent
	 */
	const VKeyboardEvent getKeyboardEvent (QKeyEvent *e);

	/**
	 * method handling the redraw
	 */
	virtual void redraw();

	/**
	 * method that sets a Pointer to the BackgroundColor
	 * @param v Ptr to the BackgroundColor
	 */
	void setBackColorPtr(VVector *v);

	/**
	 * method that sets the state of the tree layout
	 * @param layout_state is the coded value of the tree layout
	 */
	void setLayout( int layout_state );

	/**
	 * method that returns the state of the tree layout as an integer
	 */
	int getLayout();



	float m_bgColor[4];									/**< Color of Background. */
	
	TreeRep *m_TreeRep;									/**< Encasules the tree data structer and its properties. */
	
	bool *m_splinesel_active;							/**< A pointer to the boolean state of the selection enable of splines. */

	QTextEdit* name;	/**< Pointer to the text boxes where the info will be displayed. */
	QTextEdit* born;	/**< Pointer to the text boxes where the info will be displayed. */
	QTextEdit* died;	/**< Pointer to the text boxes where the info will be displayed. */
	QTextEdit* parents;	/**< Pointer to the text boxes where the info will be displayed. */
	QTextEdit* kids;	/**< Pointer to the text boxes where the info will be displayed. */
	QFotoCanvas* foto;	/**< Pointer to the cqnvqs where the foto will be displayed. */

protected:

	/**
	* method to initialise opengl
	*/
	virtual void initializeGL();

	/**
	* method called when repainting
	*/
	virtual void paintGL();


private:

	TreeLayout tLayout;						/**< Current graph layout which is active. */
	VVector *m_BackgroundColor;				/**< Current background color. */
	MouseState m_MouseState;				/**< Current Mousestate. */

	VVector m_vecPosition;					/**< Vector containing the position. */
	VVector m_PanOffset;					/**< Vector containing the pan offset. */
	float m_ZoomFactor;						/**< Vector containing the zoom factor. */

	VVector m_LineStart;					/**< Vector containing the starting position of the line. */
	VVector m_LineEnd;						/**< Vector containing the ending position of the intersection line. */

};
#endif
